@@ -34,6 +34,7 @@
 #include <xkbcommon/xkbcommon-keysyms.h>
 #include "conf.h"
 #include "kmscon_conf.h"
+#include "shl_githead.h"
 #include "shl_log.h"
 #include "shl_misc.h"
 #include "uterm_video.h"
@@ -61,7 +62,8 @@ static void print_help()
 		"given multiple times, only the last argument matters if not otherwise stated.\n"
 		"\n"
 		"General Options:\n"
-		"\t-h, --help                  [off]   Print this help and exit\n"
+		"\t-h, --help                          Print this help and exit\n"
+		"\t-V, --version                       Print version information and exit\n"
 		"\t-v, --verbose               [off]   Print verbose messages\n"
 		"\t    --debug                 [off]   Enable debug mode\n"
 		"\t    --silent                [off]   Suppress notices and warnings\n"
@@ -88,7 +90,7 @@ static void print_help()
 		"\t                              argv to this process. No more options\n"
 		"\t                              after '--' will be parsed so use it at\n"
 		"\t                              the end of the argument string\n"
-		"\t-t, --term <TERM>           [xterm-256color]\n"
+		"\t-t, --term <TERM>           [linux]\n"
 		"\t                              Value of the TERM environment variable\n"
 		"\t                              for the child process\n"
 		"\t    --reset-env             [on]\n"
@@ -156,7 +158,7 @@ static void print_help()
 		"\t    --use-original-mode     [on]    Use original KMS video mode\n"
 		"\t    --mode <width>x<height>   [0x0]  Set the desired mode for the\n"
 		"\t                                     output. If the specified mode is\n"
-		"\t                                     not available or encounterns an\n"
+		"\t                                     not available or encounters an\n"
 		"\t                                     error, a default mode will be used.\n"
 		"\t                                     This option is incompatible with\n"
 		"\t                                     --use-original-mode.\n"
@@ -206,7 +208,7 @@ static void print_help()
 		"\t    --palette-light-magenta <color> [255,   0, 255]\n"
 		"\t                                      Light magenta in custom palette\n"
 		"\t    --palette-light-cyan <color>    [  0, 255, 255]\n"
-		"\t                                      Lighty cyan in custom palette\n"
+		"\t                                      Light cyan in custom palette\n"
 		"\t    --palette-white <color>         [255, 255, 255]\n"
 		"\t                                      White in custom palette\n"
 		"\t    --palette-foreground <color>    [229, 229, 229]\n"
@@ -561,6 +563,44 @@ static int aftercheck_help(struct conf_option *opt, int argc, char **argv, int i
 		print_help();
 		conf->exit = true;
 	}
+	return 0;
+}
+static int aftercheck_version(struct conf_option *opt, int argc, char **argv, int idx)
+{
+	struct kmscon_conf_t *conf = KMSCON_CONF_FROM_FIELD(opt->mem, version);
+
+	/* exit after printing --version information */
+	if (conf->version) {
+		fprintf(stdout, "kmscon version %s\n", shl_git_head);
+
+		/* print detailed info if verbose mode is enabled */
+		if (conf->verbose) {
+			fprintf(stdout, "Compiled: %s %s\n", __DATE__, __TIME__);
+			fprintf(stdout, "Build options:\n");
+#ifdef BUILD_ENABLE_VIDEO_DRM2D
+			fprintf(stdout, "  - DRM 2D support: enabled\n");
+#else
+			fprintf(stdout, "  - DRM 2D support: disabled\n");
+#endif
+#ifdef BUILD_ENABLE_VIDEO_DRM3D
+			fprintf(stdout, "  - DRM 3D support: enabled\n");
+#else
+			fprintf(stdout, "  - DRM 3D support: disabled\n");
+#endif
+#ifdef BUILD_ENABLE_VIDEO_FBDEV
+			fprintf(stdout, "  - FBDEV support: enabled\n");
+#else
+			fprintf(stdout, "  - FBDEV support: disabled\n");
+#endif
+#ifdef BUILD_ENABLE_FONT_PANGO
+			fprintf(stdout, "  - Pango font support: enabled\n");
+#else
+			fprintf(stdout, "  - Pango font support: disabled\n");
+#endif
+		}
+
+		conf->exit = true;
+	}
 
 	return 0;
 }
@@ -691,6 +731,8 @@ int kmscon_conf_new(struct conf_ctx **out)
 	struct conf_option options[] = {
 		/* Global Options */
 		CONF_OPTION_BOOL_FULL('h', "help", aftercheck_help, NULL, NULL, &conf->help, false),
+		CONF_OPTION_BOOL_FULL('V', "version", aftercheck_version, NULL, NULL,
+				      &conf->version, false),
 		CONF_OPTION_BOOL('v', "verbose", &conf->verbose, false),
 		CONF_OPTION_BOOL_FULL(0, "debug", aftercheck_debug, NULL, NULL, &conf->debug,
 				      false),
@@ -712,7 +754,7 @@ int kmscon_conf_new(struct conf_ctx **out)
 		/* Terminal Options */
 		CONF_OPTION(0, 'l', "login", &conf_login, aftercheck_login, NULL, file_login,
 			    &conf->login, false),
-		CONF_OPTION_STRING('t', "term", &conf->term, "xterm-256color"),
+		CONF_OPTION_STRING('t', "term", &conf->term, "linux"),
 		CONF_OPTION_BOOL(0, "reset-env", &conf->reset_env, true),
 		CONF_OPTION_BOOL(0, "backspace-delete", &conf->backspace_delete, true),
 		CONF_OPTION_UINT(0, "sb-size", &conf->sb_size, 1000),
