@@ -30,93 +30,61 @@
 #include <errno.h>
 #include <fcntl.h>
 #include <inttypes.h>
+#include <libdrm/drm_fourcc.h>
 #include <stdbool.h>
+#include <stdint.h>
 #include <stdlib.h>
 #include <string.h>
 #include <sys/mman.h>
+#include <sys/types.h>
 #include <unistd.h>
 #include <xf86drm.h>
 #include <xf86drmMode.h>
 #include "eloop.h"
 #include "shl_log.h"
 #include "shl_misc.h"
-#include "uterm_drm_shared_internal.h"
 #include "uterm_drm2d_internal.h"
+#include "uterm_drm_shared_internal.h"
 #include "uterm_video.h"
 #include "uterm_video_internal.h"
 
 #define LOG_SUBSYSTEM "video_drm2d"
 
-static int display_init(struct uterm_display *disp)
+static int drm_addfb2(int fd, uint32_t width, uint32_t height, struct uterm_drm2d_rb *rb)
 {
-	struct uterm_drm2d_display *d2d;
-	int ret;
+	uint32_t handles[4] = {rb->handle, 0, 0, 0};
+	uint32_t pitches[4] = {rb->stride, 0, 0, 0};
+	uint32_t offsets[4] = {0, 0, 0, 0};
 
-	d2d = malloc(sizeof(*d2d));
-	if (!d2d)
-		return -ENOMEM;
-	memset(d2d, 0, sizeof(*d2d));
-
-	ret = uterm_drm_display_init(disp, d2d);
-	if (ret) {
-		free(d2d);
-		return ret;
-	}
-
-	return 0;
+	return drmModeAddFB2(fd, width, height, DRM_FORMAT_XRGB8888, handles, pitches, offsets,
+			     &rb->id, 0);
 }
 
-static void display_destroy(struct uterm_display *disp)
-{
-	free(uterm_drm_display_get_data(disp));
-	uterm_drm_display_destroy(disp);
-}
-
-static int init_rb(struct uterm_display *disp, struct uterm_drm2d_rb *rb)
+static int init_rb(int fd, uint32_t width, uint32_t height, struct uterm_drm2d_rb *rb)
 {
+	uint64_t mmap_offset;
 	int ret, r;
-	struct uterm_video *video = disp->video;
-	struct uterm_drm_video *vdrm = video->data;
-	struct drm_mode_create_dumb req;
-	struct drm_mode_destroy_dumb dreq;
-	struct drm_mode_map_dumb mreq;
 
-	memset(&req, 0, sizeof(req));
-	req.width = uterm_drm_mode_get_width(disp->current_mode);
-	req.height = uterm_drm_mode_get_height(disp->current_mode);
-	req.bpp = 32;
-	req.flags = 0;
-
-	ret = drmIoctl(vdrm->fd, DRM_IOCTL_MODE_CREATE_DUMB, &req);
-	if (ret < 0) {
+	if (drmModeCreateDumbBuffer(fd, width, height, 32, 0, &rb->handle, &rb->stride,
+				    &rb->size)) {
 		log_err("cannot create dumb drm buffer");
 		return -EFAULT;
 	}
 
-	rb->handle = req.handle;
-	rb->stride = req.pitch;
-	rb->size = req.size;
-
-	ret = drmModeAddFB(vdrm->fd, req.width, req.height,
-			   24, 32, rb->stride, rb->handle, &rb->fb);
+	ret = drm_addfb2(fd, width, height, rb);
 	if (ret) {
 		log_err("cannot add drm-fb");
 		ret = -EFAULT;
 		goto err_buf;
 	}
 
-	memset(&mreq, 0, sizeof(mreq));
-	mreq.handle = rb->handle;
-
-	ret = drmIoctl(vdrm->fd, DRM_IOCTL_MODE_MAP_DUMB, &mreq);
+	ret = drmModeMapDumbBuffer(fd, rb->handle, &mmap_offset);
 	if (ret) {
-		log_err("cannot map dumb buffer");
-		ret = -EFAULT;
+		log_err("Cannot map dumb buffer");
 		goto err_fb;
 	}
 
-	rb->map = mmap(0, rb->size, PROT_READ | PROT_WRITE, MAP_SHARED,
-		       vdrm->fd, mreq.offset);
+	rb->map = mmap(0, rb->size, PROT_READ | PROT_WRITE, MAP_SHARED, fd, mmap_offset);
 	if (rb->map == MAP_FAILED) {
 		log_err("cannot mmap dumb buffer");
 		ret = -EFAULT;
@@ -127,148 +95,129 @@ static int init_rb(struct uterm_display *disp, struct uterm_drm2d_rb *rb)
 	return 0;
 
 err_fb:
-	drmModeRmFB(vdrm->fd, rb->fb);
+	drmModeRmFB(fd, rb->id);
 err_buf:
-	memset(&dreq, 0, sizeof(dreq));
-	dreq.handle = rb->handle;
-	r = drmIoctl(vdrm->fd, DRM_IOCTL_MODE_DESTROY_DUMB, &dreq);
+	r = drmModeDestroyDumbBuffer(fd, rb->handle);
 	if (r)
-		log_warning("cannot destroy dumb buffer (%d/%d): %m",
-			    ret, errno);
+		log_warning("cannot destroy dumb buffer (%d/%d): %m", r, errno);
 
+	rb->size = 0;
 	return ret;
 }
 
-static void destroy_rb(struct uterm_display *disp, struct uterm_drm2d_rb *rb)
+static void destroy_rb(int fd, struct uterm_drm2d_rb *rb)
 {
-	struct uterm_drm_video *vdrm = disp->video->data;
-	struct drm_mode_destroy_dumb dreq;
 	int ret;
 
+	if (!rb->size)
+		return;
+
 	munmap(rb->map, rb->size);
-	drmModeRmFB(vdrm->fd, rb->fb);
-	memset(&dreq, 0, sizeof(dreq));
-	dreq.handle = rb->handle;
-	ret = drmIoctl(vdrm->fd, DRM_IOCTL_MODE_DESTROY_DUMB,
-		       &dreq);
+	drmModeRmFB(fd, rb->id);
+	ret = drmModeDestroyDumbBuffer(fd, rb->handle);
 	if (ret)
-		log_warning("cannot destroy dumb buffer (%d/%d): %m",
-			    ret, errno);
+		log_warning("cannot destroy dumb buffer (%d/%d): %m", ret, errno);
+	rb->size = 0;
 }
 
-static int display_activate(struct uterm_display *disp, struct uterm_mode *mode)
+static int display_allocfb(struct uterm_display *disp)
 {
-	struct uterm_video *video = disp->video;
-	struct uterm_drm_video *vdrm = video->data;
-	struct uterm_drm_display *ddrm = disp->data;
-	struct uterm_drm2d_display *d2d = uterm_drm_display_get_data(disp);
+	struct uterm_drm_video *vdrm = disp->video->data;
+	struct uterm_drm2d_display *d2d = disp->data;
 	int ret;
-	drmModeModeInfo *minfo;
 
-	if (!mode)
-		return -EINVAL;
+	disp->width = d2d->ddrm.current_mode->hdisplay;
+	disp->height = d2d->ddrm.current_mode->vdisplay;
 
-	minfo = uterm_drm_mode_get_info(mode);;
-	log_info("activating display %p to %ux%u", disp,
-		 minfo->hdisplay, minfo->vdisplay);
+	d2d->current_rb = 0;
 
-	ret = uterm_drm_display_activate(disp, vdrm->fd);
+	ret = init_rb(vdrm->fd, disp->width, disp->height, &d2d->rb[0]);
 	if (ret)
 		return ret;
 
-	d2d->current_rb = 0;
-	disp->current_mode = mode;
-
-	ret = init_rb(disp, &d2d->rb[0]);
+	ret = init_rb(vdrm->fd, disp->width, disp->height, &d2d->rb[1]);
 	if (ret)
-		goto err_saved;
-
-	ret = init_rb(disp, &d2d->rb[1]);
-	if (ret)
-		goto err_rb;
-
-	ret = drmModeSetCrtc(vdrm->fd, ddrm->crtc_id,
-			     d2d->rb[0].fb, 0, 0, &ddrm->conn_id, 1,
-			     minfo);
-
-	if (ret && mode == disp->desired_mode && mode != disp->default_mode) {
-		mode = disp->desired_mode = disp->default_mode;
-		ret = -EAGAIN;
-		goto err_fb;
-	} else if (ret) {
-		log_err("cannot set drm-crtc");
-		ret = -EFAULT;
-		goto err_fb;
-	}
-
-	disp->flags |= DISPLAY_ONLINE;
+		goto free_rb0;
 	return 0;
 
-err_fb:
-	destroy_rb(disp, &d2d->rb[1]);
-err_rb:
-	destroy_rb(disp, &d2d->rb[0]);
-err_saved:
-	disp->current_mode = NULL;
-	uterm_drm_display_deactivate(disp, vdrm->fd);
-	return ret;
+free_rb0:
+	destroy_rb(vdrm->fd, &d2d->rb[0]);
+	return 0;
 }
 
-static void display_deactivate(struct uterm_display *disp)
+static void display_freefb(struct uterm_display *disp)
 {
-	struct uterm_drm_video *vdrm;
-	struct uterm_drm2d_display *d2d = uterm_drm_display_get_data(disp);
+	struct uterm_drm_video *vdrm = disp->video->data;
+	struct uterm_drm2d_display *d2d = disp->data;
 
-	vdrm = disp->video->data;
-	log_info("deactivating display %p", disp);
-
-	uterm_drm_display_deactivate(disp, vdrm->fd);
-
-	destroy_rb(disp, &d2d->rb[1]);
-	destroy_rb(disp, &d2d->rb[0]);
-	disp->current_mode = NULL;
+	destroy_rb(vdrm->fd, &d2d->rb[0]);
+	destroy_rb(vdrm->fd, &d2d->rb[1]);
 }
 
-static int display_use(struct uterm_display *disp, bool *opengl)
+static int display_prepare_modeset(struct uterm_display *disp, drmModeAtomicReqPtr req)
 {
-	struct uterm_drm2d_display *d2d = uterm_drm_display_get_data(disp);
+	struct uterm_drm_video *vdrm = disp->video->data;
+	struct uterm_drm2d_display *d2d = disp->data;
+	int rb, ret;
 
-	if (opengl)
-		*opengl = false;
-
-	return d2d->current_rb ^ 1;
-}
-
-static int display_get_buffers(struct uterm_display *disp,
-			       struct uterm_video_buffer *buffer,
-			       unsigned int formats)
-{
-	struct uterm_drm2d_display *d2d = uterm_drm_display_get_data(disp);
-	struct uterm_drm2d_rb *rb;
-	int i;
-
-	if (!(formats & UTERM_FORMAT_XRGB32))
-		return -EOPNOTSUPP;
-
-	for (i = 0; i < 2; ++i) {
-		rb = &d2d->rb[i];
-		buffer[i].width = uterm_drm_mode_get_width(disp->current_mode);
-		buffer[i].height = uterm_drm_mode_get_height(disp->current_mode);
-		buffer[i].stride = rb->stride;
-		buffer[i].format = UTERM_FORMAT_XRGB32;
-		buffer[i].data = rb->map;
+	if (!d2d->rb[0].size) {
+		ret = display_allocfb(disp);
+		if (ret)
+			return ret;
 	}
+	rb = d2d->current_rb ^ 1;
+
+	ret = uterm_drm_prepare_commit(vdrm->fd, &d2d->ddrm, req, d2d->rb[rb].id, disp->width,
+				       disp->height);
+	if (ret)
+		return ret;
+	return 0;
+}
+
+static void display_done_modeset(struct uterm_display *disp, int status)
+{
+	struct uterm_drm2d_display *d2d = disp->data;
+	if (status) {
+		display_freefb(disp);
+	} else {
+		d2d->current_rb = d2d->current_rb ^ 1;
+	}
+}
+
+static int display_init(struct uterm_display *disp)
+{
+	struct uterm_drm2d_display *d2d;
+
+	d2d = malloc(sizeof(*d2d));
+	if (!d2d)
+		return -ENOMEM;
+	memset(d2d, 0, sizeof(*d2d));
+
+	disp->data = d2d;
+
+	d2d->ddrm.prepare_modeset = display_prepare_modeset;
+	d2d->ddrm.done_modeset = display_done_modeset;
 
 	return 0;
 }
 
-static int display_swap(struct uterm_display *disp, bool immediate)
+static void display_destroy(struct uterm_display *disp)
 {
-	int ret, rb;
-	struct uterm_drm2d_display *d2d = uterm_drm_display_get_data(disp);
+	struct uterm_drm2d_display *d2d = disp->data;
+
+	display_freefb(disp);
+	uterm_drm_display_free_properties(disp);
+	free(d2d);
+}
+
+static int display_swap(struct uterm_display *disp)
+{
+	struct uterm_drm2d_display *d2d = disp->data;
+	int ret;
+	int rb;
 
 	rb = d2d->current_rb ^ 1;
-	ret = uterm_drm_display_swap(disp, d2d->rb[rb].fb, immediate);
+	ret = uterm_drm_display_swap(disp, d2d->rb[rb].id);
 	if (ret)
 		return ret;
 
@@ -279,14 +228,15 @@ static int display_swap(struct uterm_display *disp, bool immediate)
 static const struct display_ops drm2d_display_ops = {
 	.init = display_init,
 	.destroy = display_destroy,
-	.activate = display_activate,
-	.deactivate = display_deactivate,
 	.set_dpms = uterm_drm_display_set_dpms,
-	.use = display_use,
-	.get_buffers = display_get_buffers,
+	.use = NULL,
 	.swap = display_swap,
+	.is_swapping = uterm_drm_is_swapping,
+	.need_redraw = uterm_drm_display_need_redraw,
 	.fake_blendv = uterm_drm2d_display_fake_blendv,
 	.fill = uterm_drm2d_display_fill,
+	.set_damage = uterm_drm_display_set_damage,
+	.has_damage = uterm_drm_display_has_damage,
 };
 
 static void show_displays(struct uterm_video *video)
@@ -299,7 +249,8 @@ static void show_displays(struct uterm_video *video)
 	if (!video_is_awake(video))
 		return;
 
-	shl_dlist_for_each(i, &video->displays) {
+	shl_dlist_for_each(i, &video->displays)
+	{
 		iter = shl_dlist_entry(i, struct uterm_display, list);
 
 		if (!display_is_online(iter))
@@ -313,7 +264,7 @@ static void show_displays(struct uterm_video *video)
 		 * tearing but that's acceptable as this is only called during
 		 * wakeup/sleep. */
 
-		d2d = uterm_drm_display_get_data(iter);
+		d2d = iter->data;
 		rb = &d2d->rb[d2d->current_rb];
 		memset(rb->map, 0, rb->size);
 		uterm_drm_display_wait_pflip(iter);
@@ -326,16 +277,14 @@ static int video_init(struct uterm_video *video, const char *node)
 	uint64_t has_dumb;
 	struct uterm_drm_video *vdrm;
 
-	ret = uterm_drm_video_init(video, node, &drm2d_display_ops,
-				   NULL, NULL);
+	ret = uterm_drm_video_init(video, node, &drm2d_display_ops, NULL, NULL);
 	if (ret)
 		return ret;
 	vdrm = video->data;
 
 	log_debug("initialize 2D layer on %p", video);
 
-	if (drmGetCap(vdrm->fd, DRM_CAP_DUMB_BUFFER, &has_dumb) < 0 ||
-	    !has_dumb) {
+	if (drmGetCap(vdrm->fd, DRM_CAP_DUMB_BUFFER, &has_dumb) < 0 || !has_dumb) {
 		log_err("driver does not support dumb buffers");
 		uterm_drm_video_destroy(video);
 		return -EOPNOTSUPP;
@@ -378,11 +327,12 @@ static int video_wake_up(struct uterm_video *video)
 struct uterm_video_module drm2d_module = {
 	.name = "drm2d",
 	.owner = NULL,
-	.ops = {
-		.init = video_init,
-		.destroy = video_destroy,
-		.segfault = NULL, /* TODO: reset all saved CRTCs on segfault */
-		.poll = video_poll,
-		.sleep = video_sleep,
-		.wake_up = video_wake_up,},
+	.ops =
+		{
+			.init = video_init,
+			.destroy = video_destroy,
+			.poll = video_poll,
+			.sleep = video_sleep,
+			.wake_up = video_wake_up,
+		},
 };
